# flake8: noqa

# This file is autogenerated by /metadata-ingestion/scripts/avro_codegen.py
# Do not modify manually!

# pylint: skip-file
# fmt: off
from .....schema_classes import CorpGroupEditableInfoClass
from .....schema_classes import CorpGroupInfoClass
from .....schema_classes import CorpUserAppearanceSettingsClass
from .....schema_classes import CorpUserCredentialsClass
from .....schema_classes import CorpUserEditableInfoClass
from .....schema_classes import CorpUserInfoClass
from .....schema_classes import CorpUserSettingsClass
from .....schema_classes import CorpUserStatusClass
from .....schema_classes import GroupMembershipClass
from .....schema_classes import InviteTokenClass
from .....schema_classes import NativeGroupMembershipClass
from .....schema_classes import RoleMembershipClass


CorpGroupEditableInfo = CorpGroupEditableInfoClass
CorpGroupInfo = CorpGroupInfoClass
CorpUserAppearanceSettings = CorpUserAppearanceSettingsClass
CorpUserCredentials = CorpUserCredentialsClass
CorpUserEditableInfo = CorpUserEditableInfoClass
CorpUserInfo = CorpUserInfoClass
CorpUserSettings = CorpUserSettingsClass
CorpUserStatus = CorpUserStatusClass
GroupMembership = GroupMembershipClass
InviteToken = InviteTokenClass
NativeGroupMembership = NativeGroupMembershipClass
RoleMembership = RoleMembershipClass

# fmt: on
